package com.nokia.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.nokia.dao.CityDao;
import com.nokia.dao.UserDao;
import com.nokia.dao.UserRoleDao;
import com.nokia.pojo.City;
import com.nokia.pojo.Role;
import com.nokia.pojo.User;
import com.nokia.pojo.UserRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
public class UserService extends ServiceImpl<UserDao, User> {

    private final UserDao userDao;
    private final UserRoleDao userRoleDao;
    private final CityDao cityDao;

    @Autowired
    public UserService(UserDao userDao, UserRoleDao userRoleDao, CityDao cityDao) {
        this.userDao = userDao;
        this.userRoleDao = userRoleDao;
        this.cityDao = cityDao;
    }

    @Transactional
    public String add(User table2User) {
        User query = new User();
        query.setLoginName(table2User.getLoginName());
        User one = baseMapper.selectOne(Wrappers.query(query));
        if (one != null) {
            return "loginName: " + table2User.getLoginName() + " 已存在";
        } else {
            table2User.setCityId(table2User.getCityId());
            baseMapper.insert(table2User);
            Set<Role> roleEntityList1 = table2User.getRoles();
            UserRole userRoleEntity = new UserRole();
            for (Role role : roleEntityList1) {
                userRoleEntity.setUserId(table2User.getUserId());
                userRoleEntity.setRoleId(role.getRoleId());
                userRoleDao.insert(userRoleEntity);
            }
        }
        return "ok";
    }

    @Transactional
    public String delete(Integer userId) {
        // UserEntity userEntity = baseMapper.selectById(userId);
        /* cityDao.deleteById(userEntity.getCityId()); */
        QueryWrapper<UserRole> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRole> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        for (UserRole userRoleEntity : userRoleEntities) {
            Integer id = userRoleEntity.getId();
            userRoleDao.deleteById(id);
        }
        baseMapper.deleteById(userId);
        return "ok";
    }

    /**
     * 更新用户
     * 
     * @param user
     * @return
     */
    @Transactional
    public boolean update(User user) {
        // 更新user表
        baseMapper.updateById(user);
        // 更新user_role 分为2步
        Integer userId = user.getUserId();
        QueryWrapper<UserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        // 1. 删除原user_role
        userRoleDao.delete(queryWrapper);
        // 2. 添加新的user_role
        for (Role role : user.getRoles()) {
            UserRole userRoleEntity = new UserRole();
            userRoleEntity.setUserId(user.getUserId());
            userRoleEntity.setRoleId(role.getRoleId());
            userRoleDao.insert(userRoleEntity);
        }
        return true;
    }

    /**
     * 给用户添加权限
     * 
     * @param userEntity
     * @param roleEntity
     * @return
     */
    public String addRole(User userEntity, Role roleEntity) {
        if (!hasRole(userEntity, roleEntity)) {
            // 检查用户，确认当前无此权限
            UserRole userRoleEntity = new UserRole();
            userRoleEntity.setUserId(userEntity.getUserId());
            userRoleEntity.setRoleId(roleEntity.getRoleId());
            userRoleDao.insert(userRoleEntity);
        }
        return "ok";
    }

    /**
     * 通过用户的登录名获取用户完整信息
     * 
     * @param loginName 用户登录名
     * @return UserEntity
     */
    public User getByLoginName(String loginName) {
        return userDao.getByLoinName(loginName);
    }

    /**
     * 判断用户是否有有某个角色
     * 
     * @param userEntity
     * @param roleEntity
     * @return boolean
     */
    public boolean hasRole(User userEntity, Role roleEntity) {
        return !(null == userRoleDao.hasRole(userEntity.getUserId(), roleEntity.getRoleId()));
    }

    public Map<String, Object> select(Integer userId) {
        Map<String, Object> hashMap = new HashMap<>();
        if (userId == null) {
            hashMap.put("userId", "为空");
            return hashMap;
        }
        User userEntity = baseMapper.selectById(userId);
        QueryWrapper<User> userEntityQueryWrapper = new QueryWrapper<>();
        userEntityQueryWrapper.eq("user_id", userId);
        User one = baseMapper.selectOne(userEntityQueryWrapper);
        if (one == null) {
            hashMap.put("userId", "不正确");
            return hashMap;
        }
        Integer cityId = userEntity.getCityId();
        City cityEntity = cityDao.selectById(cityId);
        // 他有两个角色 对 我知道 用for遍历来查 不需要for遍历查
        QueryWrapper<UserRole> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRole> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        hashMap.put("userEntity", userEntity);
        hashMap.put("cityEntity", cityEntity);
        hashMap.put("userRoleEntities", userRoleEntities);
        return hashMap;
    }

}
